/*===============================================================================================
 Light Grow Audio!
 Tom Stratford 2007.
 
 This code demonstrates the microphone input for the main lightGrow game.
 In the creation of the code I refered to the fmod documentation as well as Rob Bateman's
 'fmod frequencies example' (found on http://www.robthebloke.org/) and the fmod examples 'recording',
 'record to disc' and 'pitch detection'. These can be found in the fmod download package.
===============================================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include <GL/glut.h>
#include <fmod.h>
#include <fmod_errors.h>
#include <string.h>
#include <math.h>

// From FMOD example "pitchdetection"
#define OUTPUTRATE          44100
#define SPECTRUMSIZE        8192
#define SPECTRUMRANGE       ((float)OUTPUTRATE / 2.0f)      /* 0 to nyquist */
#define BINSIZE      (SPECTRUMRANGE / (float)SPECTRUMSIZE)

// Move height receptor and listener globals
float pitch, volume, volumeLast, a;

// FMOD Errorcheck -------------------------------------------------------------
void ERRCHECK(FMOD_RESULT result)
{
     if (result != FMOD_OK)
     {
        printf("FMOD error! (%d) %s\n", result, FMOD_ErrorString(result));
        exit(-1);
     }
}

// Draw Function ---------------------------------------------------------------
void OnDraw(float *wave, float maxVolNow, float maxVolThen, float freqNow, float freqThen, Uint32 timeNow, int numSeconds)
{
	// Calculate the interpolation coefficient 'a'
	a = (timeNow-numSeconds);
	a = a/1000;
	// Ranging from 0 to 1 every 0.5 seconds
	a = (a - 0.5) * 2;
	// If 'a' is negative make it the correct positive value
	if(a<0)
	{
           a = 1 + a;
    }
	// Calculate the linear interpolation
	volumeLast = volume;	
	volume = maxVolThen + (a*(maxVolNow - maxVolThen));
	pitch = freqThen + (a*(freqNow - freqThen));
    
    glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);
	glLoadIdentity();

    // Camera transformation
    glTranslatef(0,0,-5);
    glRotatef(30,1,0,0);
    glRotatef(45,0,1,0);
				
    // Draw height receptor!
    glPushMatrix();		
		glTranslatef(0,volume,0);
		glRotatef(pitch,0,1,0);
		// draw inner cube
		glColor3f(1.0,1.0,1.0);
		glutSolidCube(0.2);
        // draw outer cube			
		glScalef(1.1,1.1,1.1);
		glColor3f(0.0,0.0,0.0);
		glutWireCube(0.2);			
	glPopMatrix();
	
	// Set grid colour
	glColor3f(1.0,1.0,1.0);
	
	// Draw grid (Rob Bateman)
	glBegin(GL_LINES);
	for(int i=-5;i<=5;i++) {
		glVertex3f(i,0,-5);
		glVertex3f(i,0,5);
		glVertex3f(5,0,i);
		glVertex3f(-5,0,i);
	}
	glEnd();
	
    // Draw wave data ( 102.4 = 512/5 )
	glBegin(GL_LINE_STRIP);
	for(int i=0;i<512;i++) 
	{
        glColor3f(1.0-4*wave[i],4*wave[i],0);
        glVertex3f(((i/102.4)-2),wave[i],0);
	}
	glEnd();
	
	SDL_GL_SwapBuffers();
}
// -----------------------------------------------------------------------------



// Main ------------------------------------------------------------------------
int main(int argc,char** argv)
{
    printf("===============================================================================================\n");
    printf("Light Grow Audio Demo.\n");
    printf("Tom Stratford.\n");
    printf("===============================================================================================\n\n");    
    
    // Volume and frequency variables  
    float maxVolNow = 0;
    float maxVolThen = 0;
    float freqNow = 0;
    float freqThen = 0;
    float volumeAverage = 0;
    float frequencyAverage = 0;
    float updateTimer = 0;
    int numSeconds = 0;
    int numValues = 0; // Used for calculating the average volume and frequency
    
    
    
    
    // Initialise FMOD ---------------------------------------------------------
    FMOD_SYSTEM *fmodSystem = 0;
    FMOD_SOUND *sound = 0;
    FMOD_CHANNEL *channel = 0;
    FMOD_RESULT result;
	FMOD_CREATESOUNDEXINFO exinfo;
		
	// Create the main system object
    result = FMOD_System_Create(&fmodSystem);
    ERRCHECK(result);
     
    // Initialise FMOD to default settings
	result = FMOD_System_Init(fmodSystem, 100, FMOD_INIT_NORMAL, 0);
	ERRCHECK(result);
   
	memset(&exinfo, 0, sizeof(FMOD_CREATESOUNDEXINFO));
	
	exinfo.cbsize           = sizeof(FMOD_CREATESOUNDEXINFO);
	exinfo.numchannels      = 1;
	exinfo.format           = FMOD_SOUND_FORMAT_PCM16;
	exinfo.defaultfrequency = OUTPUTRATE;
	exinfo.length           = 11025;

	result = FMOD_System_CreateSound(fmodSystem, 0, FMOD_2D | FMOD_SOFTWARE | FMOD_LOOP_NORMAL | FMOD_OPENUSER, &exinfo, &sound);
	ERRCHECK(result);
	
	unsigned int *length;
	result = FMOD_Sound_GetLength(sound, length, FMOD_TIMEUNIT_MS);

	result = FMOD_System_RecordStart(fmodSystem, sound, true);
	ERRCHECK(result);

	result = FMOD_System_PlaySound(fmodSystem, FMOD_CHANNEL_REUSE, sound, false, &channel);
	ERRCHECK(result);
	
	result = FMOD_Channel_SetVolume(channel, 0.0);
	ERRCHECK(result);



	
	// Initialise SDL ----------------------------------------------------------
	SDL_Event event;
  
    SDL_Init(SDL_INIT_VIDEO);
    SDL_WM_SetCaption  ("lightGrow", "lightGrow.ico");
    SDL_SetVideoMode(640, 480, 0, SDL_OPENGL | SDL_HWSURFACE);
  
    Uint32 timeNow;
    Uint32 timeThen = 0;
    
    

    // Set up OpenGL -----------------------------------------------------------
    glViewport(0, 0, 640, 480);  
    glShadeModel(GL_SMOOTH);
	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	gluPerspective(45,1,0.1,100);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	
	
    // Main loop ---------------------------------------------------------------
    bool done = false;    
    while(!done)
    {                                                
        while(SDL_PollEvent(&event)) 
        {
            switch(event.type){
                case SDL_QUIT: done = true;
            break;}
        }

        // Variables for calculating volume
        float wave[512];
        float maxVol = 0;

        // Variables for calculating rough frequency
        float spectrum[SPECTRUMSIZE];
        float maxFreq = 0;
        float dominanthz = 0;
        int bin;      
        
        // Sudden loud detection variable
        int scare = 0;
        
        // Get wave data
        result = FMOD_Channel_GetWaveData(channel,wave,512,0);
        ERRCHECK(result);
        
        // Get max volume of current sample
        for(int i=0;i!=512;i++)
        {                        
            if(wave[i]>maxVol)
            {
                maxVol = wave[i];
            }
        }
        
        // Get spectum data
        result = FMOD_Channel_GetSpectrum(channel,spectrum,SPECTRUMSIZE,0,FMOD_DSP_FFT_WINDOW_TRIANGLE);
        ERRCHECK(result);           

        // Get max frequency of sample
        for (int i=0;i<8191;i++)
        {
            if (spectrum[i] > 0.01f && spectrum[i] > maxFreq)
            {
                maxFreq = spectrum[i];
                bin = i;
            }
        }
        
        // Calculate dominant frequency
        dominanthz  = (float)bin * BINSIZE;

        volumeAverage+=maxVol;
        frequencyAverage+=dominanthz;
        numValues+=1;

        timeNow = SDL_GetTicks();
        
        // Update numSeconds - numSeconds shows how many seconds have
        // passed since the program was launched.
        if(timeNow>=(numSeconds+1000))
        {
            numSeconds+=1000;
        }
        
        // Update maxVol values every 0.5 seconds
        if(timeNow>=updateTimer+500)
        {
            volumeAverage = (volumeAverage/numValues);
            frequencyAverage = (frequencyAverage/numValues);             
                                                        
            maxVolThen = maxVolNow;
            maxVolNow = volumeAverage;
            freqThen = freqNow;
            freqNow += frequencyAverage;
            
            // try to keep the value of freqThen and freqNow relatively low
            if(freqThen>=360)
            {
                freqThen-=360;
                freqNow-=360;
            }
            
            if((maxVolNow-maxVolThen)>0.45)
            {
                scare+=1;
            }
            
            volumeAverage = 0.0;
            frequencyAverage = 0.0;
            numValues = 0;           
            updateTimer += 500;
        }
                
        // Draw the scene
        OnDraw(wave, maxVolNow, maxVolThen, freqNow, freqThen, timeNow, numSeconds);
                                                  
        // Update FMOD    
        FMOD_System_Update(fmodSystem);      
    }



    // Exit FMOD
	result = FMOD_Sound_Release(sound);
	ERRCHECK(result);
    result = FMOD_System_Close(fmodSystem);
    ERRCHECK(result);
    result = FMOD_System_Release(fmodSystem);
    ERRCHECK(result);

    // Exit SDL
    SDL_Quit();
	
	return 0;
}
